@@ -15,6 +15,30 @@ cpp_code <- function(in_call, in_envir, indent = "\n    ", statement = FALSE, ex
         return(x)
     }
 
+    # Does this call produce a scalar value?
+    value_is_scalar <- function (c_val) {
+        # Single numeric values are constants
+        if (is.numeric(c_val)) return(length(c_val) == 1)
+
+        # Single parameters are constants
+        if (is.call(c_val) && c_val[[1]] == 'g3_cpp_asis' && isTRUE(c_val$scalar)) return(TRUE)
+
+        # If a variable, try fetching it out of environment and inspecting that
+        if (is.symbol(c_val) && exists(as.character(c_val), envir = in_envir, inherits = TRUE)) {
+            env_defn <- get(as.character(c_val), envir = in_envir, inherits = TRUE)
+            if (!is.null(attr(env_defn, "g3_global_init_val"))) {
+                # When considering a global formula, consider the init condition
+                env_defn <- attr(env_defn, "g3_global_init_val")
+            }
+            return(is.numeric(env_defn) && !is.array(env_defn) && length(env_defn) == 1)
+        }
+
+        # TODO: Obviously not exhaustive, but ideally one would consider setConstant() a TMB bug.
+
+        # Dunno. Assume not.
+        return(FALSE)
+    }
+
     if (!is.call(in_call)) {
         # Literals
         if (length(in_call) == 1) {
@@ -96,20 +120,6 @@ cpp_code <- function(in_call, in_envir, indent = "\n    ", statement = FALSE, ex
         assign_lhs <- in_call[[2]]
         assign_rhs <- in_call[[3]]
 
-        # Is this value a scalar?
-        value_is_scalar <- function (c_val) {
-            # Single numeric values are constants
-            if (is.numeric(c_val)) return(length(c_val) == 1)
-
-            # Single parameters are constants
-            if (is.call(c_val) && c_val[[1]] == 'g3_cpp_asis' && isTRUE(c_val$scalar)) return(TRUE)
-
-            # TODO: Obviously not exhaustive, but ideally one would consider this a TMB bug.
-
-            # Dunno. Assume not.
-            return(FALSE)
-        }
-
         # Are we assigning to an array-like object?
         if (is.call(assign_lhs) && assign_lhs[[1]] == '[') {
             if (grepl('.transpose()', cpp_code(assign_lhs, in_envir, next_indent))) {
@@ -196,8 +206,9 @@ cpp_code <- function(in_call, in_envir, indent = "\n    ", statement = FALSE, ex
         # for(x in seq(..)) loop, can expressed as a 3-part for loop
         seq_call <- call_args[[2]]
         if (is.null(seq_call$by)) stop("'by' is required in a for(seq(0, 10, by = 1)) loop: ", deparse(in_call)[[1]])
-        check_operator <- if (seq_call$by > 0) " <= " else " >= "
-        iterate_operator <- if (seq_call$by == 1) "++" else if (seq_call$by == -1) "--" else sprintf(" += %d", cpp_code(seq_call$by, in_envir, next_indent, expecting_int = TRUE))
+        by <- eval(seq_call$by, envir = baseenv())  # Convert code (read: quote(-1) ) back to numeric value
+        check_operator <- if (by > 0) " <= " else " >= "
+        iterate_operator <- if (by == 1) "++" else if (by == -1) "--" else sprintf(" += %d", cpp_code(by, in_envir, next_indent, expecting_int = TRUE))
         return(paste0(
             "for (",
             "auto ", cpp_escape_varname(call_args[[1]]), " = ", cpp_code(seq_call[[2]], in_envir, next_indent, expecting_int = TRUE), "; ",
@@ -453,16 +464,12 @@ cpp_code <- function(in_call, in_envir, indent = "\n    ", statement = FALSE, ex
     }
 
     if (call_name %in% c("is.nan")) {
-        if (is.symbol(in_call[[2]])) {
-            env_defn <- mget(as.character(in_call[[2]]), envir = in_envir, inherits = TRUE, ifnotfound = list(NA))[[1]]
-            if (!is.null(attr(env_defn, 'g3_global_init_val'))) env_defn <- attr(env_defn, 'g3_global_init_val')
-            if (is.numeric(env_defn) && length(env_defn) == 1) {
-                # Use std::isnan for single values, otherwise assume array and use Eigen method.
-                return(paste0(
-                    "std::isnan(asDouble(",
-                    cpp_code(in_call[[2]], in_envir, next_indent),
-                    "))"))
-            }
+        if ( value_is_scalar(in_call[[2]]) ) {
+            # Use std::isnan for single values, otherwise assume array and use Eigen method.
+            return(paste0(
+                "std::isnan(asDouble(",
+                cpp_code(in_call[[2]], in_envir, next_indent),
+                "))"))
         }
         return(paste0(
             "(",
@@ -471,15 +478,12 @@ cpp_code <- function(in_call, in_envir, indent = "\n    ", statement = FALSE, ex
     }
 
     if (call_name %in% c("is.finite")) {
-        if (is.symbol(in_call[[2]])) {
-            env_defn <- mget(as.character(in_call[[2]]), envir = in_envir, inherits = TRUE, ifnotfound = list(NA))[[1]]
-            if (is.numeric(env_defn) && length(env_defn) == 1) {
-                # Use std::isnan for single values, otherwise assume array and use Eigen method.
-                return(paste0(
-                    "std::isfinite(asDouble(",
-                    cpp_code(in_call[[2]], in_envir, next_indent),
-                    "))"))
-            }
+        if ( value_is_scalar(in_call[[2]]) ) {
+            # Use std::isfinite for single values, otherwise assume array and use Eigen method.
+            return(paste0(
+                "std::isfinite(asDouble(",
+                cpp_code(in_call[[2]], in_envir, next_indent),
+                "))"))
         }
         return(paste0(
             "(",
@@ -586,12 +590,19 @@ cpp_code <- function(in_call, in_envir, indent = "\n    ", statement = FALSE, ex
         paste(deparse(in_call), collapse = "\n"))
 }
 
-g3_to_tmb <- function(actions, trace = FALSE, strict = FALSE, adreport_re = '^$') {
+g3_to_tmb <- function(actions, trace = FALSE, strict = FALSE) {
     collated_actions <- g3_collate(actions)
-    all_actions <- f_concatenate(collated_actions, parent = g3_env, wrap_call = call("while", TRUE))
+    all_actions <- f_concatenate(c(
+        g3_formula(quote(cur_time <- cur_time + 1L), cur_time = -1L),
+        collated_actions,
+        NULL), parent = g3_env, wrap_call = call("while", TRUE))
     model_data <- new.env(parent = emptyenv())
     scope <- list()  # NB: Order is important, can't be an env.
 
+    # Reporting disabled by default, but updatable
+    model_data$reporting_enabled <- 0
+    scope$reporting_enabled <- 'DATA_SCALAR(reporting_enabled); DATA_UPDATE(reporting_enabled);'
+
     # Enable / disable strict mode & trace mode
     all_actions <- call_replace(all_actions,
         strict_mode = function (x) { !isFALSE(strict) },
@@ -682,7 +693,7 @@ g3_to_tmb <- function(actions, trace = FALSE, strict = FALSE, adreport_re = '^$'
 
                 # Turn table into parameter-setting definition, adding individual PARAMETERs as we go
                 init_data <- vapply(seq_len(nrow(df)), function (i) {
-                    sub_param_name <- paste0(c(as.character(x[[2]]), df[i,]), collapse = ".")
+                    sub_param_name <- gen_param_tbl_name(as.character(x[[2]]), df[i,])
                     sub_param_tuple <- paste0(df[i,], collapse = ",")
 
                     scope[[cpp_escape_varname(sub_param_name)]] <<- structure(
@@ -715,12 +726,25 @@ g3_to_tmb <- function(actions, trace = FALSE, strict = FALSE, adreport_re = '^$'
                 }
             }
 
+            if (x[[1]] == 'g3_param_lower' || x[[1]] == 'g3_param_upper') {
+                param_name <- paste0(param_name, if (x[[1]] == 'g3_param_lower') "__lower" else "__upper")
+                scope[[param_name]] <<- sprintf(
+                    "DATA_SCALAR(%s);",
+                    param_name )
+                # NB: We'll update these later with real values
+                model_data[[param_name]] <<- NaN
+                # NB: Tell assignment if we're scalar, so it can use is.finite()
+                return(call("g3_cpp_asis", param_name, scalar = TRUE))
+            }
+
             # Add PARAMETER definition for variable
-            scope[[param_name]] <<- structure(sprintf("PARAMETER%s(%s);",
-                if (x[[1]] == 'g3_param_array') '_ARRAY'
-                else if (x[[1]] == 'g3_param_vector') '_VECTOR'
-                else '',
-                param_name), param_template = df_template(x[[2]]))
+            if (x[[1]] != 'g3_param_nodef') {
+                scope[[param_name]] <<- structure(sprintf("PARAMETER%s(%s);",
+                    if (x[[1]] == 'g3_param_array') '_ARRAY'
+                    else if (x[[1]] == 'g3_param_vector') '_VECTOR'
+                    else '',
+                    param_name), param_template = df_template(x[[2]]))
+            }
             # NB: Tell assignment if we're scalar, so it can use setConstant()
             return(call("g3_cpp_asis", param_name, scalar = (x[[1]] == 'g3_param')))
         }
@@ -728,6 +752,9 @@ g3_to_tmb <- function(actions, trace = FALSE, strict = FALSE, adreport_re = '^$'
             g3_param_table = repl_fn,
             g3_param_array = repl_fn,
             g3_param_vector = repl_fn,
+            g3_param_lower = repl_fn,
+            g3_param_upper = repl_fn,
+            g3_param_nodef = repl_fn,
             g3_param = repl_fn)
 
         # Find all things that have definitions in our environment
@@ -766,9 +793,13 @@ g3_to_tmb <- function(actions, trace = FALSE, strict = FALSE, adreport_re = '^$'
                 call("stop", "Incomplete model: No definition for ", var_name)
             })
 
-            if (rlang::is_formula(var_val)) {
+            if (is.call(var_val)) {  # i.e. either bare-code or a formula
                 # Recurse, get definitions for formula, considering it's environment as well as the outer one
-                var_val_code <- var_defns(rlang::f_rhs(var_val), rlang::env_clone(rlang::f_env(var_val), parent = env))
+                if (rlang::is_formula(var_val)) {
+                    var_val_code <- var_defns(rlang::f_rhs(var_val), rlang::env_clone(rlang::f_env(var_val), parent = env))
+                } else {
+                    var_val_code <- var_defns(var_val, new.env(parent = env))
+                }
                 if (var_name %in% names(scope)) {
                     # var_name got defined as a side-effect of the above (it's a g3_param)
                     # so don't change anything
@@ -776,8 +807,6 @@ g3_to_tmb <- function(actions, trace = FALSE, strict = FALSE, adreport_re = '^$'
                 } else {
                     defn <- cpp_definition('auto', cpp_escape_varname(var_name), cpp_code(var_val_code, env))
                 }
-            } else if (is.call(var_val)) {
-                defn <- cpp_definition('auto', var_name, cpp_code(var_val, env))
             } else {
                 # Decide base type
                 if (all(is.integer(var_val))) {
@@ -851,7 +880,6 @@ g3_to_tmb <- function(actions, trace = FALSE, strict = FALSE, adreport_re = '^$'
 
             attr(defn, 'report_names') <- names(var_val)
             attr(defn, 'report_dimnames') <- dimnames(var_val)
-            attr(defn, 'report_dynamic_dimnames') <- attr(var_val, 'dynamic_dimnames')
             scope[[var_name]] <<- defn
         }
         return(code)
@@ -860,12 +888,6 @@ g3_to_tmb <- function(actions, trace = FALSE, strict = FALSE, adreport_re = '^$'
     # Define all vars, populating scope in process
     all_actions_code <- var_defns(rlang::f_rhs(all_actions), rlang::f_env(all_actions))
 
-    # Rework any g3_* function calls into the code we expect
-    g3_functions <- function (in_code) {
-        call_replace(in_code,
-            g3_report_all = function (x) g3_functions(action_reports(collated_actions, REPORT = '.', ADREPORT = adreport_re)))
-    }
-    all_actions_code <- g3_functions(all_actions_code)
     ss <- scope_split(scope)
 
 
@@ -876,7 +898,6 @@ Type objective_function<Type>::operator() () {
     %s
 
     %s
-    abort();  // Should have returned somewhere in the loop
 }\n", paste(ss[['function']], collapse = "\n"), paste(ss$definition, collapse = "\n    "),
       cpp_code(all_actions_code, rlang::f_env(all_actions), statement = TRUE))
     out <- strsplit(out, "\n")[[1]]
@@ -913,12 +934,18 @@ Type objective_function<Type>::operator() () {
     class(out) <- c("g3_cpp", class(out))
 
     attr(out, 'actions') <- actions
-    attr(out, 'model_data') <- model_data
     attr(out, 'parameter_template') <- scope_to_parameter_template(scope, 'data.frame')
+    attr(out, 'model_data') <- update_data_bounds(model_data, attr(out, 'parameter_template'))
     attr(out, 'report_renames') <- scope_to_cppnamemap(scope)
     attr(out, 'report_names') <- Filter(Negate(is.null), lapply(scope, function (x) attr(x, 'report_names')))
     attr(out, 'report_dimnames') <- Filter(Negate(is.null), lapply(scope, function (x) attr(x, 'report_dimnames')))
-    attr(out, 'report_dynamic_dimnames') <- Filter(Negate(is.null), lapply(scope, function (x) attr(x, 'report_dynamic_dimnames')))
+    attr(out, 'report_gen_dimnames') <- mget(
+        "gen_dimnames",
+        envir = rlang::f_env(all_actions),
+        ifnotfound = list(NA),
+        inherits = TRUE )[[1]]
+    # NB: ifnotfound doesn't work with function output
+    if (!is.function(attr(out, 'report_gen_dimnames'))) attr(out, 'report_gen_dimnames') <- function (x) 0
     return(out)
 }
 
@@ -1031,10 +1058,11 @@ g3_tmb_adfun <- function(cpp_code,
         dyn.load(so_path)
     }
 
+    # Update any bounds used by the model
+    tmb_data <- as.list(update_data_bounds(attr(cpp_code, 'model_data'), parameters))
     if (output_script) {
         tmp_script_path <- tempfile(fileext = ".R")
         tmp_data_path <- paste0(tmp_script_path, "data")
-        tmb_data <- attr(cpp_code, 'model_data')
         save(tmb_data, tmb_parameters, tmb_map, tmb_random, file = tmp_data_path)
         writeLines(c(
             "library(TMB)",
@@ -1042,7 +1070,7 @@ g3_tmb_adfun <- function(cpp_code,
             deparse(call("load", tmp_data_path)),
             "",
             deparse(call("MakeADFun",
-                data = as.list(tmb_data),
+                data = tmb_data,
                 parameters = quote(tmb_parameters),
                 map = quote(tmb_map),
                 random = quote(tmb_random),
@@ -1051,7 +1079,7 @@ g3_tmb_adfun <- function(cpp_code,
         return(tmp_script_path)
     }
     fn <- TMB::MakeADFun(
-        data = as.list(attr(cpp_code, 'model_data')),
+        data = tmb_data,
         parameters = tmb_parameters,
         map = tmb_map,
         random = tmb_random,
@@ -1061,9 +1089,23 @@ g3_tmb_adfun <- function(cpp_code,
     report_names <- attr(cpp_code, 'report_names')
     report_dimnames <- attr(cpp_code, 'report_dimnames')
     report_renames <- attr(cpp_code, 'report_renames')
-    report_dynamic_dimnames <- attr(cpp_code, 'report_dynamic_dimnames')
+
+    # Run gen_dimnames & repopulate any dynamic dims
+    # TODO: This should be paying attention to the code within, not just assuming that gen_dimnames was used
+    dyndims <- attributes(attr(cpp_code, 'report_gen_dimnames')(tmb_parameters))
+    for (dimname in names(dyndims)) {
+        for (var_name in names(report_dimnames)) {
+            if (dimname %in% names(report_dimnames[[var_name]]) && is.null(report_dimnames[[var_name]][[dimname]])) {
+                report_dimnames[[var_name]][[dimname]] <- dyndims[[dimname]]
+            }
+        }
+    }
+
     fn$orig_report <- fn$report
     fn$report <- function (...) {
+        old_reporting_enabled <- fn$env$data$reporting_enabled
+        fn$env$data$reporting_enabled <- 1
+        on.exit(fn$env$data$reporting_enabled <- old_reporting_enabled)
         out <- fn$orig_report(...)
         # Patch report names back again
         for (dimname in names(report_renames)) {
@@ -1088,13 +1130,6 @@ g3_tmb_adfun <- function(cpp_code,
                 out[[dimname]],
                 length(out[[dimname]]))
 
-            # Find and bodge any dynamic dims into having something that fits
-            for (di in seq_along(report_dimnames[[dimname]])) {
-                if (is.null(report_dimnames[[dimname]][[di]]) && !is.null(report_dynamic_dimnames[[dimname]][[di]])) {
-                    report_dimnames[[dimname]][[di]] <- seq_len(dim(out[[dimname]])[[di]])
-                }
-            }
-
             names(dim(out[[dimname]])) <- names(report_dimnames[[dimname]])
             dimnames(out[[dimname]]) <- report_dimnames[[dimname]]
         }
