@@ -24,6 +24,11 @@ ok_group("step_id", {
 
     ok(ut_cmp_identical(step_id(0, 10, 200), "000:010:200"), "Multiple numbers: 0, 10, 200")
     ok(ut_cmp_identical(step_id(0, stock_a, "camel"), "000:stock_aaa           :camel               "), "All the things")
+
+    ok(ut_cmp_identical(
+        sort(sapply(-1:11, gadget3:::step_id), method="radix"),
+        c("-01", "000", "001", "002", "003", "004", "005", "006", "007", "008", "009", "010", "011"),
+        filter = NULL), "-1 sorted before 0")
 })
 
 ok_group("g3_step:stock_reshape", {
@@ -47,7 +52,6 @@ ok_group("g3_step:stock_reshape", {
     dest_nolength <- gadget3:::g3_storage('dest_nolength')  # No length at all
     dest_nolength__num <- g3_stock_instance(dest_nolength)
 
-    cur_time <- 0L  # Initialconditions needs to know what the time is
     nll <- 0.0
     actions <- list(
         g3a_initialconditions(source, ~g3_param_vector("source_num"), ~g3_param_vector("source_wgt")),
@@ -267,22 +271,22 @@ ok_group("g3_step:dependent_formulas", (function () {
 
 })())
 
-ok_group("g3_step:stock_param", {
+ok_group("g3_step:stock_prepend", {
     stock_a <- g3_stock(c(t = 'stock', q = 'stick', 'aaa'), seq(10, 35, 5))
 
     ok(cmp_code(
         gadget3:::g3_step(~{
-            stock_param(stock_a, 'parr', optmise = FALSE)
-            stock_param(stock_a, 'parr', optmise = FALSE, upper = 5)
+            stock_prepend(stock_a, g3_param('parr', optmise = FALSE))
+            stock_prepend(stock_a, g3_param('parr', optmise = FALSE, upper = 5))
         }), ~{
             g3_param("stock_stick_aaa.parr", optmise = FALSE)
             g3_param("stock_stick_aaa.parr", optmise = FALSE, upper = 5)
         }), "Passed through options")
     ok(cmp_code(
         gadget3:::g3_step(~{
-            stock_param(stock_a, name_part = 't', 'parr')
-            stock_param(stock_a, 'parr', name_part = 't')
-            stock_param(stock_a, 'parr', name_part = 't', lower = 4, upper = 5)
+            stock_prepend(stock_a, name_part = 't', g3_param('parr'))
+            stock_prepend(stock_a, g3_param('parr'), name_part = 't')
+            stock_prepend(stock_a, g3_param('parr', lower = 4, upper = 5), name_part = 't')
         }), ~{
             g3_param("stock.parr")
             g3_param("stock.parr")
@@ -290,24 +294,32 @@ ok_group("g3_step:stock_param", {
         }), "name_part can be either beffore or after name, not passed through to g3_param call")
     ok(cmp_code(
         gadget3:::g3_step(~{
-            stock_param(stock_a, name_part = c('t', 'q'), 'par1')
-            stock_param(stock_a, name_part = c('q', 't'), 'par1')
-            stock_param(stock_a, name_part = c('t'), 'par1')
+            stock_prepend(stock_a, name_part = c('t', 'q'), g3_param('par1'))
+            stock_prepend(stock_a, name_part = c('q', 't'), g3_param('par1'))
+            stock_prepend(stock_a, name_part = c('t'), g3_param('par1'))
         }), ~{
             g3_param("stock_stick.par1")
             g3_param("stick_stock.par1")
             g3_param("stock.par1")
         }), "name_part can contain multiple name_parts, get used in order")
+    ok(cmp_code(
+        gadget3:::g3_step(~{
+            stock_prepend("bling", g3_param("Linf", value = 1))
+            stock_prepend("blang", g3_param("Linf", value = 1))
+        }), ~{
+            g3_param("bling.Linf", value = 1)
+            g3_param("blang.Linf", value = 1)
+        }), "stock_var can also be a string (worked out by g3_parameterized), which just gets prepended")
 })
 
-ok_group("g3_step:stock_param_table", {
+ok_group("g3_step:stock_prepend:table", {
     stock_a <- g3_stock(c(t = 'stock', 'aaa'), seq(10, 35, 5))
     stock_b <- g3_stock(c(t = 'stock', 'bbb'), seq(10, 35, 5))
 
     ok(cmp_code(
         gadget3:::g3_step(~{
-            stock_param_table(stock_a, 'par1', data.frame(age = seq(stock_a__minage, stock_a__maxage), year = 2:3))
-            stock_param_table(stock_a, 'par1', data.frame(age = seq(stock_a__minage, stock_a__maxage), len = stock_a__minlen))
+            stock_prepend(stock_a, g3_param_table('par1', data.frame(age = seq(stock_a__minage, stock_a__maxage), year = 2:3)))
+            stock_prepend(stock_a, g3_param_table('par1', data.frame(age = seq(stock_a__minage, stock_a__maxage), len = stock_a__minlen)))
         }), ~{
             g3_param_table("stock_aaa.par1", data.frame(age = seq(stock_aaa__minage, stock_aaa__maxage), year = 2:3))
             g3_param_table("stock_aaa.par1", data.frame(age = seq(stock_aaa__minage, stock_aaa__maxage), len = stock_aaa__minlen))
@@ -315,8 +327,8 @@ ok_group("g3_step:stock_param_table", {
 
     ok(cmp_code(
         gadget3:::g3_step(~{
-            stock_param_table(stock_a, 'par1', data.frame(year = 2:3), upper = 5)
-            stock_param_table(stock_a, 'par1', data.frame(year = 2:3), upper = 5, lower = 2)
+            stock_prepend(stock_a, g3_param_table('par1', data.frame(year = 2:3), upper = 5))
+            stock_prepend(stock_a, g3_param_table('par1', data.frame(year = 2:3), upper = 5, lower = 2))
         }), ~{
             g3_param_table('stock_aaa.par1', data.frame(year = 2:3), upper = 5)
             g3_param_table('stock_aaa.par1', data.frame(year = 2:3), upper = 5, lower = 2)
@@ -324,8 +336,8 @@ ok_group("g3_step:stock_param_table", {
 
     ok(cmp_code(
         gadget3:::g3_step(~{
-            stock_param_table(stock_a, name_part = 't', 'par1', data.frame(year = 2:3), upper = 5)
-            stock_param_table(stock_a, 'par1', name_part = 't', data.frame(year = 2:3), upper = 5, lower = 2)
+            stock_prepend(stock_a, name_part = 't', g3_param_table('par1', data.frame(year = 2:3), upper = 5))
+            stock_prepend(stock_a, g3_param_table('par1', data.frame(year = 2:3), upper = 5, lower = 2), name_part = 't')
         }), ~{
             g3_param_table('stock.par1', data.frame(year = 2:3), upper = 5)
             g3_param_table('stock.par1', data.frame(year = 2:3), upper = 5, lower = 2)
@@ -333,8 +345,8 @@ ok_group("g3_step:stock_param_table", {
 
     ok(cmp_code(
         gadget3:::g3_step(~{
-            stock_param_table(stock_a, name_part = 't', 'par1', stock_with(stock_b, data.frame(age = seq(stock_a__minage, stock_b__maxage))))
-            stock_param_table(stock_b, name_part = 't', 'par1', stock_with(stock_a, data.frame(age = seq(stock_a__minage, stock_b__maxage))))
+            stock_prepend(stock_a, name_part = 't', g3_param_table('par1', stock_with(stock_b, data.frame(age = seq(stock_a__minage, stock_b__maxage)))))
+            stock_prepend(stock_b, name_part = 't', g3_param_table('par1', stock_with(stock_a, data.frame(age = seq(stock_a__minage, stock_b__maxage)))))
         }), ~{
             g3_param_table('stock.par1', data.frame(age = seq(stock_aaa__minage, stock_bbb__maxage)))
             g3_param_table('stock.par1', data.frame(age = seq(stock_aaa__minage, stock_bbb__maxage)))
@@ -343,9 +355,9 @@ ok_group("g3_step:stock_param_table", {
     stock_a <- g3_stock(c(t = 'stock', q = 'stick', 'aaa'), seq(10, 35, 5))
     ok(cmp_code(
         gadget3:::g3_step(~{
-            stock_param_table(stock_a, name_part = c('t', 'q'), 'par1', data.frame(year = 2:3))
-            stock_param_table(stock_a, name_part = c('q', 't'), 'par1', data.frame(year = 2:3))
-            stock_param_table(stock_a, name_part = c('t'), 'par1', data.frame(year = 2:3))
+            stock_prepend(stock_a, name_part = c('t', 'q'), g3_param_table('par1', data.frame(year = 2:3)))
+            stock_prepend(stock_a, name_part = c('q', 't'), g3_param_table('par1', data.frame(year = 2:3)))
+            stock_prepend(stock_a, name_part = c('t'), g3_param_table('par1', data.frame(year = 2:3)))
         }), ~{
             g3_param_table("stock_stick.par1", data.frame(year = 2:3))
             g3_param_table("stick_stock.par1", data.frame(year = 2:3))
